(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common')) :
	typeof define === 'function' && define.amd ? define('miappio-sdk', ['exports', '@angular/core', '@angular/common'], factory) :
	(factory((global['miappio-sdk'] = {}),global.ng.core,global.ng.common));
}(this, (function (exports,core,common) { 'use strict';

var Base64 = /** @class */ (function () {
    function Base64() {
    }
    Base64.encode = function (input) {
        if (!input) {
            return null;
        }
        return btoa(encodeURIComponent(input).replace(/%([0-9A-F]{2})/g, function toSolidBytes(match, p1) {
            return String.fromCharCode(parseInt('0x' + p1, 16));
        }));
    };
    Base64.decode = function (input) {
        if (!input) {
            return null;
        }
        return decodeURIComponent(atob(input).split('').map(function (c) {
            return '%' + ('00' + c.charCodeAt(0).toString(16)).slice(-2);
        }).join(''));
    };
    return Base64;
}());
var LocalStorage = /** @class */ (function () {
    function LocalStorage(storageService, storageKey) {
        this.storageKey = storageKey;
        this.version = '0.1';
        this.storage = storageService || window.localStorage;
        if (!this.storage) {
            throw new Error('miapp.LocalStorageFactory needs a storageService!');
        }
    }
    LocalStorage.prototype.set = function (key, value) {
        key = this.storageKey + key;
        this.checkKey(key);
        var t = typeof (value);
        if (t === 'undefined') {
            value = 'null';
        }
        else if (value === null) {
            value = 'null';
        }
        else if (t === 'string') {
            value = JSON.stringify({ string: value });
        }
        else if (t === 'number') {
            value = JSON.stringify({ number: value });
        }
        else if (t === 'boolean') {
            value = JSON.stringify({ bool: value });
        }
        else if (t === 'object') {
            value = JSON.stringify({ json: value });
        }
        else {
            throw new TypeError('Value type ' + t + ' is invalid. It must be null, undefined, xml, string, number, boolean or object');
        }
        this.storage.setItem(key, value);
        return value;
    };
    LocalStorage.prototype.get = function (key, def) {
        key = this.storageKey + key;
        this.checkKey(key);
        var item = this.storage.getItem(key);
        if (item !== null) {
            if (item === 'null') {
                return null;
            }
            var value = JSON.parse(item);
            if ('string' in value) {
                return value.string;
            }
            else if ('number' in value) {
                return value.number.valueOf();
            }
            else if ('bool' in value) {
                return value.bool.valueOf();
            }
            else {
                return value.json;
            }
        }
        return !def ? null : def;
    };
    LocalStorage.prototype.remove = function (key) {
        key = this.storageKey + key;
        this.checkKey(key);
        var existed = (this.storage.getItem(key) !== null);
        this.storage.removeItem(key);
        return existed;
    };
    LocalStorage.prototype.clear = function () {
        var existed = (this.storage.length > 0);
        this.storage.clear();
        return existed;
    };
    LocalStorage.prototype.size = function () {
        return this.storage.length;
    };
    LocalStorage.prototype.foreach = function (f, context) {
        var n = this.storage.length;
        for (var i = 0; i < n; i++) {
            var key = this.storage.key(i);
            var value = this.get(key);
            if (context) {
                f.call(context, value);
            }
            else {
                f(value);
            }
        }
        return n;
    };
    LocalStorage.prototype.checkKey = function (key) {
        if (!key || (typeof key !== 'string')) {
            throw new TypeError('Key type must be string');
        }
        return true;
    };
    return LocalStorage;
}());
var Xor = /** @class */ (function () {
    function Xor() {
    }
    Xor.encrypt = function (value, key) {
        var result = '';
        value = Xor.header + value;
        for (var i = 0; i < value.length; i++) {
            result += String.fromCharCode(((value[i].charCodeAt(0).toString(10))) ^ Xor.keyCharAt(key, i));
        }
        result = Base64.encode(result);
        return result;
    };
    Xor.decrypt = function (value, key, oldStyle) {
        var result = '';
        value = Base64.decode(value);
        for (var i = 0; i < value.length; i++) {
            result += String.fromCharCode(((value[i].charCodeAt(0).toString(10))) ^ Xor.keyCharAt(key, i));
        }
        if (!oldStyle && Xor.header !== result.substring(0, Xor.header.length)) {
            return null;
        }
        if (!oldStyle) {
            result = result.substring(Xor.header.length);
        }
        return result;
    };
    Xor.keyCharAt = function (key, i) {
        return key[Math.floor(i % key.length)].charCodeAt(0).toString(10);
    };
    return Xor;
}());
Xor.header = 'artemis-lotsum';
var version = '2.0.45';
var XHRPromise = /** @class */ (function () {
    function XHRPromise() {
        this.DEFAULT_CONTENT_TYPE = 'application/x-www-form-urlencoded; charset=UTF-8';
    }
    XHRPromise.prototype.send = function (options) {
        var defaults;
        if (options == null) {
            options = {};
        }
        defaults = {
            method: 'GET',
            data: null,
            headers: {},
            async: true,
            username: null,
            password: null,
            withCredentials: false
        };
        options = Object.assign({}, defaults, options);
        return new Promise((function (_this) {
            return function (resolve, reject) {
                var e, header, ref, value, xhr;
                if (!XMLHttpRequest) {
                    _this._handleError('browser', reject, null, 'browser doesn\'t support XMLHttpRequest');
                    return;
                }
                if (typeof options.url !== 'string' || options.url.length === 0) {
                    _this._handleError('url', reject, null, 'URL is a required parameter');
                    return;
                }
                _this._xhr = xhr = new XMLHttpRequest;
                xhr.onload = function () {
                    var responseText;
                    _this._detachWindowUnload();
                    try {
                        responseText = _this._getResponseText();
                    }
                    catch (_error) {
                        _this._handleError('parse', reject, null, 'invalid JSON response');
                        return;
                    }
                    return resolve({
                        url: _this._getResponseUrl(),
                        status: xhr.status,
                        statusText: xhr.statusText,
                        responseText: responseText,
                        headers: _this._getHeaders(),
                        xhr: xhr
                    });
                };
                xhr.onerror = function () {
                    return _this._handleError('error', reject);
                };
                xhr.ontimeout = function () {
                    return _this._handleError('timeout', reject);
                };
                xhr.onabort = function () {
                    return _this._handleError('abort', reject);
                };
                _this._attachWindowUnload();
                xhr.open(options.method, options.url, options.async, options.username, options.password);
                if (options.withCredentials) {
                    xhr.withCredentials = true;
                }
                if ((options.data != null) && !options.headers['Content-Type']) {
                    options.headers['Content-Type'] = _this.DEFAULT_CONTENT_TYPE;
                }
                ref = options.headers;
                for (header in ref) {
                    if (ref.hasOwnProperty(header)) {
                        value = ref[header];
                        xhr.setRequestHeader(header, value);
                    }
                }
                try {
                    return xhr.send(options.data);
                }
                catch (_error) {
                    e = _error;
                    return _this._handleError('send', reject, null, e.toString());
                }
            };
        })(this));
    };
    XHRPromise.prototype.getXHR = function () {
        return this._xhr;
    };
    XHRPromise.prototype._attachWindowUnload = function () {
        this._unloadHandler = this._handleWindowUnload.bind(this);
        if (((window)).attachEvent) {
            return ((window)).attachEvent('onunload', this._unloadHandler);
        }
    };
    XHRPromise.prototype._detachWindowUnload = function () {
        if (((window)).detachEvent) {
            return ((window)).detachEvent('onunload', this._unloadHandler);
        }
    };
    XHRPromise.prototype._getHeaders = function () {
        return this._parseHeaders(this._xhr.getAllResponseHeaders());
    };
    XHRPromise.prototype._getResponseText = function () {
        var responseText;
        responseText = typeof this._xhr.responseText === 'string' ? this._xhr.responseText : '';
        switch ((this._xhr.getResponseHeader('Content-Type') || '').split(';')[0]) {
            case 'application/json':
            case 'text/javascript':
                responseText = JSON.parse(responseText + '');
        }
        return responseText;
    };
    XHRPromise.prototype._getResponseUrl = function () {
        if (this._xhr.responseURL != null) {
            return this._xhr.responseURL;
        }
        if (/^X-Request-URL:/m.test(this._xhr.getAllResponseHeaders())) {
            return this._xhr.getResponseHeader('X-Request-URL');
        }
        return '';
    };
    XHRPromise.prototype._handleError = function (reason, reject, status, statusText) {
        this._detachWindowUnload();
        var code = 404;
        if (reason === 'timeout') {
            code = 408;
        }
        else if (reason === 'abort') {
            code = 408;
        }
        return reject({
            reason: reason,
            status: status || this._xhr.status || code,
            code: status || this._xhr.status || code,
            statusText: statusText || this._xhr.statusText,
            xhr: this._xhr
        });
    };
    XHRPromise.prototype._handleWindowUnload = function () {
        return this._xhr.abort();
    };
    XHRPromise.prototype.trim = function (str) {
        return str.replace(/^\s*|\s*$/g, '');
    };
    XHRPromise.prototype.isArray = function (arg) {
        return Object.prototype.toString.call(arg) === '[object Array]';
    };
    XHRPromise.prototype.forEach = function (list, iterator) {
        if (toString.call(list) === '[object Array]') {
            this.forEachArray(list, iterator, this);
        }
        else if (typeof list === 'string') {
            this.forEachString(list, iterator, this);
        }
        else {
            this.forEachObject(list, iterator, this);
        }
    };
    XHRPromise.prototype.forEachArray = function (array, iterator, context) {
        for (var i = 0, len = array.length; i < len; i++) {
            if (array.hasOwnProperty(i)) {
                iterator.call(context, array[i], i, array);
            }
        }
    };
    XHRPromise.prototype.forEachString = function (string, iterator, context) {
        for (var i = 0, len = string.length; i < len; i++) {
            iterator.call(context, string.charAt(i), i, string);
        }
    };
    XHRPromise.prototype.forEachObject = function (object, iterator, context) {
        for (var k in object) {
            if (object.hasOwnProperty(k)) {
                iterator.call(context, object[k], k, object);
            }
        }
    };
    XHRPromise.prototype._parseHeaders = function (headers) {
        var _this = this;
        if (!headers) {
            return {};
        }
        var result = {};
        this.forEach(this.trim(headers).split('\n'), function (row) {
            var index = row.indexOf(':'), key = _this.trim(row.slice(0, index)).toLowerCase(), value = _this.trim(row.slice(index + 1));
            if (typeof (result[key]) === 'undefined') {
                result[key] = value;
            }
            else if (_this.isArray(result[key])) {
                result[key].push(value);
            }
            else {
                result[key] = [result[key], value];
            }
        });
        return result;
    };
    return XHRPromise;
}());
var Ajax = /** @class */ (function () {
    function Ajax() {
        this.xhr = new XHRPromise();
    }
    Ajax.prototype.post = function (args) {
        var opt = {
            method: 'POST',
            url: args.url,
            data: JSON.stringify(args.data)
        };
        if (args.headers) {
            opt.headers = args.headers;
        }
        return this.xhr
            .send(opt)
            .then(function (res) {
            if (res.status &&
                (parseInt(res.status, 10) < 200 || parseInt(res.status, 10) >= 300)) {
                res.reason = 'status';
                res.code = parseInt(res.status, 10);
                return Promise.reject(res);
            }
            return Promise.resolve(res.responseText);
        })
            .catch(function (err) {
            return Promise.reject(err);
        });
    };
    Ajax.prototype.put = function (args) {
        var opt = {
            method: 'PUT',
            url: args.url,
            data: JSON.stringify(args.data)
        };
        if (args.headers) {
            opt.headers = args.headers;
        }
        return this.xhr
            .send(opt)
            .then(function (res) {
            if (res.status &&
                (parseInt(res.status, 10) < 200 || parseInt(res.status, 10) >= 300)) {
                res.reason = 'status';
                res.code = parseInt(res.status, 10);
                return Promise.reject(res);
            }
            return Promise.resolve(res.responseText);
        })
            .catch(function (err) {
            return Promise.reject(err);
        });
    };
    Ajax.prototype.delete = function (args) {
        var opt = {
            method: 'DELETE',
            url: args.url,
            data: JSON.stringify(args.data)
        };
        if (args.headers) {
            opt.headers = args.headers;
        }
        return this.xhr
            .send(opt)
            .then(function (res) {
            if (res.status &&
                (parseInt(res.status, 10) < 200 || parseInt(res.status, 10) >= 300)) {
                res.reason = 'status';
                res.code = parseInt(res.status, 10);
                return Promise.reject(res);
            }
            return Promise.resolve(res.responseText);
        })
            .catch(function (err) {
            return Promise.reject(err);
        });
    };
    Ajax.prototype.get = function (args) {
        var opt = {
            method: 'GET',
            url: args.url
        };
        if (args.data) {
            opt.data = args.data;
        }
        if (args.headers) {
            opt.headers = args.headers;
        }
        return this.xhr
            .send(opt)
            .then(function (res) {
            if (res.status &&
                (parseInt(res.status, 10) < 200 || parseInt(res.status, 10) >= 300)) {
                res.reason = 'status';
                res.code = parseInt(res.status, 10);
                return Promise.reject(res);
            }
            return Promise.resolve(res.responseText);
        })
            .catch(function (err) {
            return Promise.reject(err);
        });
    };
    return Ajax;
}());
var Client = /** @class */ (function () {
    function Client(appId, URI, storage, sdk) {
        this.appId = appId;
        this.URI = URI;
        this.storage = storage;
        this.sdk = sdk;
        var uuid = this.storage.get(Client._clientUuid) || 'uuid-' + Math.random();
        var info = '_clientInfo';
        if (window && window.navigator) {
            info = window.navigator.appName + '@' + window.navigator.appVersion + '-' + window.navigator.userAgent;
        }
        if (window && window['device'] && window['device'].uuid) {
            uuid = window['device'].uuid;
        }
        this.setClientUuid(uuid);
        this.setClientInfo(info);
        this.clientId = this.storage.get(Client._clientId);
        Client.refreshCount = this.storage.get(Client._refreshCount) || 0;
    }
    Client.prototype.setClientId = function (value) {
        this.clientId = '' + value;
        this.storage.set(Client._clientId, this.clientId);
    };
    Client.prototype.setClientUuid = function (value) {
        this.clientUuid = '' + value;
        this.storage.set(Client._clientUuid, this.clientUuid);
    };
    Client.prototype.setClientInfo = function (value) {
        this.clientInfo = '' + value;
    };
    Client.prototype.login = function (login, password, updateProperties) {
        var _this = this;
        if (!this.URI) {
            console.error('no api uri');
            return Promise.reject({ code: 408, reason: 'no-api-uri' });
        }
        var urlLogin = this.URI + '/users';
        var dataLogin = {
            name: login,
            username: login,
            email: login,
            password: password
        };
        return new Ajax()
            .post({
            url: urlLogin,
            data: dataLogin,
            headers: { 'Content-Type': 'application/json', 'Accept': 'text/json' }
        })
            .then(function (createdUser) {
            _this.setClientId(createdUser._id);
            var urlToken = _this.URI + '/oauth/token';
            var dataToken = {
                grant_type: 'client_credentials',
                client_id: _this.clientId,
                client_secret: password,
                client_udid: _this.clientUuid,
                client_info: _this.clientInfo,
                audience: _this.appId,
                scope: JSON.stringify(_this.sdk)
            };
            return new Ajax()
                .post({
                url: urlToken,
                data: dataToken,
                headers: { 'Content-Type': 'application/json', 'Accept': 'text/json' }
            });
        });
    };
    Client.prototype.reAuthenticate = function (refreshToken) {
        var _this = this;
        if (!this.URI) {
            console.error('no api uri');
            return Promise.reject({ code: 408, reason: 'no-api-uri' });
        }
        var url = this.URI + '/oauth/token';
        var data = {
            grant_type: 'refresh_token',
            client_id: this.clientId,
            client_udid: this.clientUuid,
            client_info: this.clientInfo,
            audience: this.appId,
            scope: JSON.stringify(this.sdk),
            refresh_token: refreshToken,
            refresh_extra: Client.refreshCount,
        };
        return new Ajax()
            .post({
            url: url,
            data: data,
            headers: { 'Content-Type': 'application/json', 'Accept': 'text/json' }
        })
            .then(function (obj) {
            Client.refreshCount++;
            _this.storage.set(Client._refreshCount, Client.refreshCount);
            return Promise.resolve(obj);
        });
    };
    Client.prototype.logout = function (refreshToken) {
        if (!this.URI) {
            console.error('no api uri');
            return Promise.reject({ code: 408, reason: 'no-api-uri' });
        }
        this.storage.remove(Client._clientUuid);
        this.storage.remove(Client._clientId);
        this.storage.remove(Client._refreshCount);
        Client.refreshCount = 0;
        if (!refreshToken || !this.clientId) {
            return Promise.resolve();
        }
        var url = this.URI + '/oauth/revoke';
        var data = {
            token: refreshToken,
            client_id: this.clientId,
            client_udid: this.clientUuid,
            client_info: this.clientInfo,
            audience: this.appId,
            scope: JSON.stringify(this.sdk)
        };
        return new Ajax()
            .post({
            url: url,
            data: data,
            headers: { 'Content-Type': 'application/json', 'Accept': 'text/json' }
        });
    };
    Client.prototype.isReady = function () {
        return !!this.URI;
    };
    return Client;
}());
Client.refreshCount = 0;
Client._clientUuid = 'v2.clientUuid';
Client._clientId = 'v2.clientId';
Client._refreshCount = 'v2.refreshCount';
var Error$1 = /** @class */ (function () {
    function Error$1(code, reason) {
        this.code = code;
        this.reason = reason;
    }
    Error$1.prototype.equals = function (err) {
        return this.code === err.code && this.reason === err.reason;
    };
    return Error$1;
}());
var Connection = /** @class */ (function () {
    function Connection(_sdk, _storage) {
        this._sdk = _sdk;
        this._storage = _storage;
        this.client = null;
        this.user = null;
        this.cryptoSalt = this._storage.get(Connection._cryptoSalt) || null;
        this.cryptoSaltNext = this._storage.get(Connection._cryptoSaltNext) || null;
        this.accessToken = this._storage.get(Connection._accessToken) || null;
        this.accessTokenPrevious = this._storage.get('v2.accessTokenPrevious') || null;
        this.idToken = this._storage.get(Connection._idToken) || null;
        this.refreshToken = this._storage.get(Connection._refreshToken) || null;
        this.states = this._storage.get(Connection._states) || {};
        this.apis = [];
    }
    Connection.prototype.isReady = function () {
        return !!this.client && this.client.isReady();
    };
    Connection.prototype.destroy = function (force) {
        this._storage.remove(Connection._accessToken);
        this._storage.remove(Connection._idToken);
        this._storage.remove(Connection._refreshToken);
        this._storage.remove(Connection._states);
        if (this.accessToken) {
            this.accessTokenPrevious = this.accessToken;
            this._storage.set(Connection._accessTokenPrevious, this.accessTokenPrevious);
        }
        if (force) {
            this._storage.remove(Connection._cryptoSalt);
            this._storage.remove(Connection._cryptoSaltNext);
            this._storage.remove(Connection._accessTokenPrevious);
        }
        this.user = null;
        if (this.client) {
            this.client.logout();
        }
        this.accessToken = null;
        this.idToken = null;
        this.refreshToken = null;
        this.states = {};
    };
    Connection.prototype.setClient = function (client) {
        this.client = client;
        if (!this.user) {
            this.user = {};
        }
        this.user._name = JSON.parse(this.getIdPayload({ name: '' })).name;
    };
    Connection.prototype.setUser = function (user) {
        this.user = user;
        if (this.user._id) {
            this.client.setClientId(this.user._id);
            delete this.user._id;
        }
    };
    Connection.prototype.getUser = function () {
        return this.user;
    };
    Connection.prototype.getClient = function () {
        return this.client;
    };
    Connection.prototype.setCryptoSalt = function (value) {
        if (this.cryptoSalt !== value && this.cryptoSaltNext !== value) {
            this.cryptoSaltNext = value;
            this._storage.set(Connection._cryptoSaltNext, this.cryptoSaltNext);
        }
        if (!this.cryptoSalt) {
            this.setCryptoSaltAsVerified();
        }
    };
    Connection.prototype.setCryptoSaltAsVerified = function () {
        if (this.cryptoSaltNext) {
            this.cryptoSalt = this.cryptoSaltNext;
            this._storage.set(Connection._cryptoSalt, this.cryptoSalt);
        }
        this.cryptoSaltNext = null;
        this._storage.remove(Connection._cryptoSaltNext);
    };
    Connection.prototype.encrypt = function (data) {
        if (typeof data !== 'string') {
            data = JSON.stringify(data);
        }
        else {
            var dataAsObj = { string: data };
            data = JSON.stringify(dataAsObj);
        }
        if (this.miappCrypto && this.cryptoSalt) {
            var key = this.cryptoSalt;
            return Xor.encrypt(data, key);
        }
        else {
            return data;
        }
    };
    Connection.prototype.decrypt = function (data) {
        var decrypted = null;
        try {
            if (!decrypted && this.miappCrypto && this.cryptoSaltNext) {
                var key = this.cryptoSaltNext;
                decrypted = Xor.decrypt(data, key);
                decrypted = JSON.parse(decrypted);
            }
        }
        catch (err) {
            decrypted = null;
        }
        try {
            if (!decrypted && this.miappCrypto && this.cryptoSalt) {
                var key = this.cryptoSalt;
                decrypted = Xor.decrypt(data, key);
                decrypted = JSON.parse(decrypted);
            }
        }
        catch (err) {
            decrypted = null;
        }
        try {
            if (!decrypted && this.miappCrypto && this.cryptoSalt) {
                var key = this.cryptoSalt;
                decrypted = Xor.decrypt(data, key, true);
                decrypted = JSON.parse(decrypted);
            }
        }
        catch (err) {
            decrypted = null;
        }
        try {
            if (!decrypted) {
                decrypted = JSON.parse(data);
            }
            if (decrypted && decrypted.string) {
                decrypted = decrypted.string;
            }
        }
        catch (err) {
            decrypted = null;
        }
        return decrypted;
    };
    Connection.prototype.isLogin = function () {
        var exp = true;
        try {
            var payload = this.refreshToken.split('.')[1];
            var decoded = JSON.parse(Base64.decode(payload));
            exp = ((new Date().getTime() / 1000) >= decoded.exp);
        }
        catch (e) {
        }
        return !exp;
    };
    Connection.prototype.logout = function () {
        return this.getClient().logout(this.refreshToken);
    };
    Connection.prototype.getClientId = function () {
        if (!this.client) {
            return null;
        }
        return this.client.clientId;
    };
    Connection.prototype.getIdPayload = function (def) {
        if (def && typeof def !== 'string') {
            def = JSON.stringify(def);
        }
        try {
            var payload = this.idToken.split('.')[1];
            if (payload) {
                return Base64.decode(payload);
            }
        }
        catch (e) {
        }
        return def ? def : null;
    };
    Connection.prototype.getAccessPayload = function (def) {
        if (def && typeof def !== 'string') {
            def = JSON.stringify(def);
        }
        try {
            var payload = this.accessToken.split('.')[1];
            if (payload) {
                return Base64.decode(payload);
            }
        }
        catch (e) {
        }
        return def ? def : null;
    };
    Connection.prototype.getPreviousAccessPayload = function (def) {
        if (def && typeof def !== 'string') {
            def = JSON.stringify(def);
        }
        try {
            var payload = this.accessTokenPrevious.split('.')[1];
            if (payload) {
                return Base64.decode(payload);
            }
        }
        catch (e) {
        }
        return def ? def : null;
    };
    Connection.prototype.refreshConnection = function () {
        var _this = this;
        this._storage.set(Connection._states, this.states);
        if (this.accessToken) {
            var payload = this.accessToken.split('.')[1];
            var decoded = Base64.decode(payload);
            if ((new Date().getTime() / 1000) < JSON.parse(decoded).exp) {
                return Promise.resolve(this.getUser());
            }
        }
        if (this.refreshToken) {
            var payload = this.refreshToken.split('.')[1];
            var decoded = Base64.decode(payload);
            if ((new Date().getTime() / 1000) >= JSON.parse(decoded).exp) {
                this._storage.remove(Connection._refreshToken);
            }
        }
        this.accessTokenPrevious = this.accessToken;
        this._storage.set('v2.accessTokenPrevious', this.accessTokenPrevious);
        this._storage.remove(Connection._accessToken);
        this._storage.remove(Connection._idToken);
        this.accessToken = null;
        this.idToken = null;
        return new Promise(function (resolve, reject) {
            _this.getClient().reAuthenticate(_this.refreshToken)
                .then(function (user) {
                _this.setConnection(user);
                resolve(_this.getUser());
            })
                .catch(function (err) {
                reject(err);
            });
        });
    };
    Connection.prototype.setConnection = function (clientUser) {
        if (clientUser.access_token) {
            this.accessToken = clientUser.access_token;
            this._storage.set(Connection._accessToken, this.accessToken);
            delete clientUser.access_token;
            var salt = JSON.parse(this.getAccessPayload({ salt: '' })).salt;
            if (salt) {
                this.setCryptoSalt(salt);
            }
        }
        if (clientUser.id_token) {
            this.idToken = clientUser.id_token;
            this._storage.set(Connection._idToken, this.idToken);
            delete clientUser.id_token;
        }
        if (clientUser.refresh_token) {
            this.refreshToken = clientUser.refresh_token;
            this._storage.set(Connection._refreshToken, this.refreshToken);
            delete clientUser.refresh_token;
        }
        this._storage.set(Connection._states, this.states);
        clientUser.roles = JSON.parse(this.getIdPayload({ roles: [] })).roles;
        clientUser.message = JSON.parse(this.getIdPayload({ message: '' })).message;
        this.setUser(clientUser);
    };
    Connection.prototype.setConnectionOffline = function (options) {
        if (options.accessToken) {
            this.accessToken = options.accessToken;
            this._storage.set(Connection._accessToken, this.accessToken);
        }
        if (options.idToken) {
            this.idToken = options.idToken;
            this._storage.set(Connection._idToken, this.idToken);
        }
        if (options.refreshToken) {
            this.refreshToken = options.refreshToken;
            this._storage.set(Connection._refreshToken, this.refreshToken);
        }
        this.setUser({
            roles: JSON.parse(this.getIdPayload({ roles: [] })).roles,
            message: JSON.parse(this.getIdPayload({ message: '' })).message,
            _id: 'demo'
        });
    };
    Connection.prototype.getApiEndpoints = function (options) {
        var ea = ['https://miapp.io/api'];
        var filteredEa = [];
        if (!this._sdk.prod) {
            ea = ['http://localhost:5894/api', 'https://miapp-sandbox.herokuapp.com/api'];
        }
        if (this.accessToken) {
            var val = this.getAccessPayload({ apis: [] });
            var apiEndpoints = JSON.parse(val).apis;
            if (apiEndpoints && apiEndpoints.length) {
                ea = [];
                apiEndpoints.forEach(function (endpoint) {
                    ea.push(endpoint);
                });
            }
        }
        if (this.accessTokenPrevious) {
            var apiEndpoints = JSON.parse(this.getPreviousAccessPayload({ apis: [] })).apis;
            if (apiEndpoints && apiEndpoints.length) {
                apiEndpoints.forEach(function (endpoint) {
                    if (ea.indexOf(endpoint) < 0) {
                        ea.push(endpoint);
                    }
                });
            }
        }
        var couldCheckStates = true;
        if (this.states && Object.keys(this.states).length) {
            for (var i = 0; (i < ea.length) && couldCheckStates; i++) {
                if (!this.states[ea[i]]) {
                    couldCheckStates = false;
                }
            }
        }
        else {
            couldCheckStates = false;
        }
        if (options && options.filter) {
            if (couldCheckStates && options.filter === 'theBestOne') {
                for (var i = 0; (i < ea.length) && (filteredEa.length === 0); i++) {
                    var endpoint = ea[i];
                    if (this.states[endpoint] &&
                        this.states[endpoint].state) {
                        filteredEa.push(endpoint);
                    }
                }
            }
            else if (couldCheckStates && options.filter === 'theBestOldOne') {
                var bestOldOne = void 0;
                for (var i = 0; (i < ea.length); i++) {
                    var endpoint = ea[i];
                    if (this.states[endpoint] &&
                        this.states[endpoint].lastTimeWasOk &&
                        (!bestOldOne || this.states[endpoint].lastTimeWasOk > this.states[bestOldOne].lastTimeWasOk)) {
                        bestOldOne = endpoint;
                    }
                }
                if (bestOldOne) {
                    filteredEa.push(bestOldOne);
                }
            }
            else if (ea.length) {
                filteredEa.push(ea[0]);
            }
        }
        else {
            filteredEa = ea;
        }
        return filteredEa;
    };
    Connection.prototype.getDBs = function (options) {
        if (!this.accessToken) {
            return [];
        }
        var random = Math.random() % 2;
        var dbs = JSON.parse(this.getAccessPayload({ dbs: [] })).dbs || [];
        if (random === 0) {
            dbs = dbs.sort();
        }
        else if (random === 1) {
            dbs = dbs.reverse();
        }
        var filteredDBs = [];
        var couldCheckStates = true;
        if (this.states && Object.keys(this.states).length) {
            for (var i = 0; (i < dbs.length) && couldCheckStates; i++) {
                if (!this.states[dbs[i]]) {
                    couldCheckStates = false;
                }
            }
        }
        else {
            couldCheckStates = false;
        }
        if (couldCheckStates && options && options.filter === 'theBestOne') {
            for (var i = 0; (i < dbs.length) && (filteredDBs.length === 0); i++) {
                var endpoint = dbs[i];
                if (this.states[endpoint] &&
                    this.states[endpoint].state) {
                    filteredDBs.push(endpoint);
                }
            }
        }
        else if (couldCheckStates && options && options.filter === 'theBestOnes') {
            for (var i = 0; (i < dbs.length); i++) {
                var endpoint = dbs[i];
                if (this.states[endpoint] &&
                    this.states[endpoint].state) {
                    filteredDBs.push(endpoint);
                }
            }
        }
        else if (options && options.filter === 'theBestOne' && dbs.length) {
            filteredDBs.push(dbs[0]);
        }
        else {
            filteredDBs = dbs;
        }
        return filteredDBs;
    };
    Connection.prototype.verifyConnectionStates = function () {
        var _this = this;
        var currentTime = new Date().getTime();
        var promises = [];
        this.apis = this.getApiEndpoints();
        this.apis.forEach(function (endpoint) {
            promises.push(new Promise(function (resolve, reject) {
                new Ajax()
                    .get({
                    url: endpoint + '/status?isok=' + _this._sdk.version,
                    headers: { 'Content-Type': 'application/json', 'Accept': 'text/json' }
                })
                    .then(function (data) {
                    var state = false;
                    if (data && data.isok) {
                        state = true;
                    }
                    _this.states[endpoint] = { state: state, time: currentTime, lastTimeWasOk: currentTime };
                    resolve();
                })
                    .catch(function (err) {
                    var lastTimeWasOk = 0;
                    if (_this.states[endpoint]) {
                        lastTimeWasOk = _this.states[endpoint].lastTimeWasOk;
                    }
                    _this.states[endpoint] = { state: false, time: currentTime, lastTimeWasOk: lastTimeWasOk };
                    resolve();
                });
            }));
        });
        var dbs = this.getDBs();
        dbs.forEach(function (dbEndpoint) {
            promises.push(new Promise(function (resolve, reject) {
                new Ajax()
                    .get({
                    url: dbEndpoint,
                    headers: { 'Content-Type': 'application/json', 'Accept': 'text/json' }
                })
                    .then(function (data) {
                    _this.states[dbEndpoint] = { state: true, time: currentTime, lastTimeWasOk: currentTime };
                    resolve();
                })
                    .catch(function (err) {
                    var lastTimeWasOk = 0;
                    if (_this.states[dbEndpoint]) {
                        lastTimeWasOk = _this.states[dbEndpoint].lastTimeWasOk;
                    }
                    _this.states[dbEndpoint] = { state: false, time: currentTime, lastTimeWasOk: lastTimeWasOk };
                    resolve();
                });
            }));
        });
        return Promise.all(promises);
    };
    return Connection;
}());
Connection._accessToken = 'v2.accessToken';
Connection._accessTokenPrevious = 'v2.accessTokenPrevious';
Connection._idToken = 'v2.idToken';
Connection._refreshToken = 'v2.refreshToken';
Connection._states = 'v2.states';
Connection._cryptoSalt = 'v2.cryptoSalt';
Connection._cryptoSaltNext = 'v2.cryptoSalt.next';
var Pouch = window['PouchDB'] || require('pouchdb').default;
var Session = /** @class */ (function () {
    function Session() {
        this.db = null;
        this.dbRecordCount = 0;
        this.dbLastSync = null;
        this.remoteDb = null;
        this.dbs = [];
    }
    Session.prototype.isReady = function () {
        return !!this.db;
    };
    Session.prototype.create = function (force) {
        var _this = this;
        if (!force && this.db) {
            return Promise.resolve();
        }
        this.dbRecordCount = 0;
        this.dbLastSync = null;
        this.db = {};
        return new Promise(function (resolve, reject) {
            var opts = { location: 'default' };
            try {
                _this.db = new Pouch('miapp_db', opts);
                _this.db.info()
                    .then(function (info) {
                    return resolve(_this.db);
                }).catch(function (err) {
                    reject(new Error$1(400, err));
                });
            }
            catch (err) {
                reject(function (err) {
                    reject(new Error$1(500, err));
                });
            }
        });
    };
    Session.prototype.destroy = function () {
        var _this = this;
        if (!this.db) {
            this.dbRecordCount = 0;
            this.dbLastSync = null;
            return Promise.resolve();
        }
        if (this.db && !this.db.destroy) {
            return Promise.reject(new Error$1(408, 'Need a valid db'));
        }
        return new Promise(function (resolve, reject) {
            _this.db.destroy(function (err, info) {
                if (err) {
                    reject(new Error$1(500, err.toString()));
                }
                else {
                    _this.dbRecordCount = 0;
                    _this.dbLastSync = null;
                    _this.db = null;
                    resolve();
                }
            });
        });
    };
    Session.prototype.setRemote = function (dbs) {
        this.dbs = dbs;
    };
    Session.prototype.sync = function (userId) {
        var _this = this;
        if (!this.db) {
            return Promise.reject(new Error$1(408, 'need db'));
        }
        if (!this.dbs || !this.dbs.length) {
            return Promise.reject(new Error$1(408, 'need a remote db'));
        }
        return new Promise(function (resolve, reject) {
            try {
                if (!_this.remoteDb || _this.remoteUri !== _this.dbs[0]) {
                    _this.remoteUri = _this.dbs[0];
                    _this.remoteDb = new Pouch(_this.remoteUri);
                }
                _this.db.replicate.to(_this.remoteDb)
                    .on('complete', function (info) {
                    return _this.remoteDb.replicate.to(_this.db, {
                        filter: function (doc) {
                            return (!!userId && !!doc && doc.miappUserId === userId);
                        }
                    })
                        .on('complete', function () {
                        resolve();
                    })
                        .on('denied', function (err) { return reject({ code: 403, reason: err }); })
                        .on('error', function (err) { return reject({ code: 401, reason: err }); });
                })
                    .on('denied', function (err) { return reject({ code: 403, reason: err }); })
                    .on('error', function (err) { return reject({ code: 401, reason: err }); });
            }
            catch (err) {
                reject({ code: 500, reason: err });
            }
        });
    };
    Session.prototype.put = function (data, _id, uid, oid, ave, crypto) {
        var _this = this;
        if (!this.db) {
            return Promise.reject(new Error$1(408, 'need db'));
        }
        if (!data || !_id || !uid || !oid || !ave) {
            return Promise.reject(new Error$1(400, 'need formated data'));
        }
        var dataWithoutIds = JSON.parse(JSON.stringify(data));
        var toStore = {
            _id: _id,
            miappUserId: uid,
            miappOrgId: oid,
            miappAppVersion: ave
        };
        if (dataWithoutIds._rev) {
            toStore._rev = '' + dataWithoutIds._rev;
        }
        delete dataWithoutIds._id;
        delete dataWithoutIds._rev;
        delete dataWithoutIds.miappUserId;
        delete dataWithoutIds.miappOrgId;
        delete dataWithoutIds.miappAppVersion;
        delete dataWithoutIds.miappData;
        var resultAsString = Session.write(Session.value(dataWithoutIds));
        if (crypto) {
            resultAsString = crypto.obj[crypto.method](resultAsString);
            toStore.miappDacr = resultAsString;
        }
        else {
            toStore.miappData = resultAsString;
        }
        return new Promise(function (resolve, reject) {
            _this.db.put(toStore, function (err, response) {
                if (response && response.ok && response.id && response.rev) {
                    _this.dbRecordCount++;
                    if (typeof data === 'object') {
                        ((data))._rev = response.rev;
                        ((data))._id = response.id;
                        resolve(data);
                    }
                    else {
                        resolve(response.id);
                    }
                }
                else {
                    reject(new Error$1(500, err.toString()));
                }
            });
        });
    };
    Session.prototype.remove = function (data_id) {
        var _this = this;
        if (!this.db) {
            return Promise.reject(new Error$1(408, 'need db'));
        }
        return new Promise(function (resolve, reject) {
            _this.db.get(data_id)
                .then(function (doc) {
                doc._deleted = true;
                return _this.db.put(doc);
            })
                .then(function (result) {
                resolve();
            })
                .catch(function (err) {
                reject(err);
            });
        });
    };
    Session.prototype.get = function (data_id, crypto) {
        var _this = this;
        if (!this.db) {
            return Promise.reject(new Error$1(408, 'Need db'));
        }
        return new Promise(function (resolve, reject) {
            _this.db.get(data_id)
                .then(function (row) {
                if (!!row && (!!row.miappDacr || !!row.miappData)) {
                    var data = row.miappDacr;
                    if (crypto && data) {
                        data = crypto.obj[crypto.method](data);
                    }
                    else if (row.miappData) {
                        data = JSON.parse(row.miappData);
                    }
                    var resultAsJson = Session.extractJson(data);
                    if (resultAsJson) {
                        resultAsJson._id = row._id;
                        resultAsJson._rev = row._rev;
                        resolve(JSON.parse(JSON.stringify(resultAsJson)));
                    }
                    else {
                        _this.remove(row._id);
                        reject(new Error$1(400, 'Bad encoding'));
                    }
                }
                else {
                    reject(new Error$1(400, 'No data found'));
                }
            })
                .catch(function (err) { return reject(new Error$1(500, JSON.stringify(err))); });
        });
    };
    Session.prototype.getAll = function (crypto) {
        var _this = this;
        if (!this.db || !((this.db)).allDocs) {
            return Promise.reject(new Error$1(408, 'Need a valid db'));
        }
        return new Promise(function (resolve, reject) {
            ((_this.db)).allDocs({ include_docs: true, descending: true })
                .then(function (rows) {
                var all = [];
                rows.rows.forEach(function (row) {
                    if (!!row && !!row.doc._id && (!!row.doc.miappDacr || !!row.doc.miappData)) {
                        var data = row.doc.miappDacr;
                        if (crypto && data) {
                            data = crypto.obj[crypto.method](data);
                        }
                        else if (row.doc.miappData) {
                            data = JSON.parse(row.doc.miappData);
                        }
                        var resultAsJson = Session.extractJson(data);
                        if (resultAsJson) {
                            resultAsJson._id = row.doc._id;
                            resultAsJson._rev = row.doc._rev;
                            all.push(JSON.parse(JSON.stringify(resultAsJson)));
                        }
                        else {
                            console.error('Bad encoding : delete row');
                            _this.remove(row.doc._id);
                        }
                    }
                    else {
                        console.error('Bad encoding');
                    }
                });
                resolve(all);
            })
                .catch(function (err) { return reject(new Error$1(400, err)); });
        });
    };
    Session.prototype.isEmpty = function () {
        var _this = this;
        if (!this.db || !((this.db)).allDocs) {
            return Promise.reject(new Error$1(408, 'No db'));
        }
        return new Promise(function (resolve, reject) {
            ((_this.db)).allDocs({})
                .then(function (response) {
                if (!response) {
                    reject(new Error$1(400, 'No response'));
                }
                else {
                    _this.dbRecordCount = response.total_rows;
                    if (response.total_rows && response.total_rows > 0) {
                        resolve(false);
                    }
                    else {
                        resolve(true);
                    }
                }
            })
                .catch(function (err) { return reject(new Error$1(400, err)); });
        });
    };
    Session.prototype.info = function () {
        if (!this.db) {
            return Promise.reject(new Error$1(408, 'No db'));
        }
        return this.db.info();
    };
    Session.write = function (item) {
        var value = 'null';
        var t = typeof (item);
        if (t === 'undefined') {
            value = 'null';
        }
        else if (value === null) {
            value = 'null';
        }
        else if (t === 'string') {
            value = JSON.stringify({ string: item });
        }
        else if (t === 'number') {
            value = JSON.stringify({ number: item });
        }
        else if (t === 'boolean') {
            value = JSON.stringify({ bool: item });
        }
        else if (t === 'object') {
            value = JSON.stringify({ json: item });
        }
        return value;
    };
    Session.value = function (item) {
        var result = item;
        if (typeof (item) !== 'object') {
        }
        else if ('string' in item) {
            result = item.string;
        }
        else if ('number' in item) {
            result = item.number.valueOf();
        }
        else if ('bool' in item) {
            result = item.bool.valueOf();
        }
        else if ('json' in item) {
            result = item.json;
            if (typeof (result) !== 'object') {
                result = JSON.parse(result);
            }
        }
        return result;
    };
    Session.extractJson = function (item) {
        var result = item;
        if (!item) {
            return null;
        }
        if (typeof (item) === 'object' && 'json' in item) {
            result = item.json;
        }
        if (typeof (result) === 'string') {
            result = JSON.parse(result);
        }
        if (typeof (result) === 'object' && 'json' in result) {
            result = ((result)).json;
        }
        if (typeof result !== 'object') {
            result = null;
        }
        return result;
    };
    return Session;
}());
var InternalService = /** @class */ (function () {
    function InternalService(logger, promise) {
        this.sdk = {
            org: 'miapp.io',
            version: version,
            prod: false
        };
        this.logger = {
            log: function () {
            },
            error: function () {
            },
            warn: function () {
            }
        };
        if (logger && window.console && logger === window.console) {
            this.logger.error = window.console.error;
            this.logger.warn = window.console.warn;
        }
        this.logger.log('miapp.sdk.service : constructor');
        if (promise) {
            this.promise = promise;
        }
        this.storage = new LocalStorage(window.localStorage, 'miapp.');
        this.session = new Session();
        this.connection = new Connection(this.sdk, this.storage);
    }
    InternalService.prototype.miappInit = function (miappId, options) {
        var self = this;
        self.logger.log('miapp.sdk.service.miappInit : ', options);
        if (!miappId) {
            self.logger.error('miapp.sdk.service.miappInit : bad init');
            return self.promise.reject(new Error$1(400, 'Need a miappId'));
        }
        self.sdk.prod = !options ? true : options.prod;
        return new self.promise(function (resolve, reject) {
            self.connection.verifyConnectionStates()
                .then(function () {
                self.connection.miappId = miappId;
                self.connection.miappVersion = self.sdk.version;
                self.connection.miappCrypto = (!options || !options.hasOwnProperty('crypto')) ? true : options.crypto;
                var theBestUrl = self.connection.getApiEndpoints({ filter: 'theBestOne' })[0];
                var theBestOldUrl = self.connection.getApiEndpoints({ filter: 'theBestOldOne' })[0];
                var isLogin = self.miappIsLogin();
                if (theBestUrl) {
                    self.connection.setClient(new Client(self.connection.miappId, theBestUrl, self.storage, self.sdk));
                    resolve();
                }
                else if (isLogin && theBestOldUrl) {
                    self.connection.setClient(new Client(self.connection.miappId, theBestOldUrl, self.storage, self.sdk));
                    resolve();
                }
                else {
                    reject(new Error$1(404, 'Need one connection'));
                }
            })
                .catch(function (err) {
                self.logger.error('miapp.sdk.service.miappInit: ', err);
                reject(new Error$1(500, err.toString()));
            });
        });
    };
    InternalService.prototype.miappLogin = function (login, password) {
        var self = this;
        self.logger.log('miapp.sdk.service.miappLogin');
        if (!self.connection.isReady()) {
            return self.promise.reject(new Error$1(404, 'Need an intialized MiappService'));
        }
        return new self.promise(function (resolve, reject) {
            self._removeAll()
                .then(function () {
                return self.connection.verifyConnectionStates();
            })
                .then(function () {
                return self._createSession();
            })
                .then(function () {
                return self._loginInternal(login, password);
            })
                .then(function (user) {
                self.connection.setConnection(user);
                self.session.sync(self.connection.getClientId())
                    .then(function () { return resolve(self.connection.getUser()); })
                    .catch(function (err) { return resolve(self.connection.getUser()); });
            })
                .catch(function (err) {
                self.logger.error('miapp.sdk.service.miappLogin: ', err.toString());
                reject(err);
            });
        });
    };
    InternalService.prototype.miappLoginInDemoMode = function (options) {
        var self = this;
        if (!options || !options.accessToken) {
            var now = new Date();
            now.setDate(now.getDate() + 1);
            var tomorrow = now.getTime();
            var payload = Base64.encode(JSON.stringify({
                roles: [],
                message: 'demo',
                apis: [],
                endpoints: {},
                dbs: [],
                exp: tomorrow
            }));
            var jwtSign = Base64.encode(JSON.stringify({}));
            var token = jwtSign + '.' + payload + '.' + jwtSign;
            options = {
                accessToken: token,
                idToken: token,
                refreshToken: token
            };
        }
        return new self.promise(function (resolve, reject) {
            self._removeAll()
                .then(function () {
                return self._createSession();
            })
                .then(function () {
                self.connection.setConnectionOffline(options);
                resolve(self.connection.getUser());
            })
                .catch(function (err) {
                self.logger.error('miapp.sdk.service.miappLogin error: ', err);
                reject(err);
            });
        });
    };
    InternalService.prototype.miappGetEndpoints = function (options) {
        return JSON.parse(this.connection.getIdPayload({ endpoints: [] })).endpoints;
    };
    InternalService.prototype.miappRoles = function () {
        return JSON.parse(this.connection.getIdPayload({ roles: [] })).roles;
    };
    InternalService.prototype.miappMessage = function () {
        return JSON.parse(this.connection.getIdPayload({ message: '' })).message;
    };
    InternalService.prototype.miappIsLogin = function () {
        return this.connection.isLogin();
    };
    InternalService.prototype.miappLogout = function () {
        var _this = this;
        var self = this;
        if (!self.connection.getClient()) {
            return self._removeAll()
                .then(function () {
                return _this.session.create(true);
            });
        }
        return self.connection.logout()
            .then(function () {
            return self._removeAll();
        })
            .catch(function () {
            return self._removeAll();
        })
            .then(function () {
            return _this.session.create(true);
        });
    };
    InternalService.prototype.miappSync = function (fnInitFirstData, fnInitFirstData_Arg) {
        var _this = this;
        var self = this;
        self.logger.log('miapp.sdk.service.miappSync');
        var firstSync = (self.session.dbLastSync === null);
        return new self.promise(function (resolve, reject) {
            self._createSession()
                .then(function () {
                return self.session.sync(self.connection.getClientId());
            })
                .then(function () {
                self.logger.log('miapp.sdk.service.miappSync resolved');
                return self.session.isEmpty();
            })
                .catch(function (err) {
                self.logger.warn('miapp.sdk.service.miappSync warn: ', err);
                return self.session.isEmpty();
            })
                .then(function (isEmpty) {
                self.logger.log('miapp.sdk.service.miappSync isEmpty : ', isEmpty, firstSync);
                return new Promise(function (resolveEmpty, rejectEmptyNotUsed) {
                    if (isEmpty && firstSync && fnInitFirstData) {
                        var ret = fnInitFirstData(fnInitFirstData_Arg);
                        if (ret && ret['catch'] instanceof Function) {
                            ret.then(resolveEmpty).catch(reject);
                        }
                        if (typeof ret === 'string') {
                            self.logger.log(ret);
                        }
                    }
                    resolveEmpty();
                });
            })
                .then(function (info) {
                self.logger.log('miapp.sdk.service.miappSync fnInitFirstData resolved: ', info);
                self.session.dbLastSync = new Date().getTime();
                return self.session.info();
            })
                .then(function (result) {
                self.session.dbRecordCount = 0;
                if (result && result.doc_count) {
                    self.session.dbRecordCount = result.doc_count;
                }
                self.logger.log('miapp.sdk.service.miappSync _dbRecordCount : ' + self.session.dbRecordCount);
                return self.connection.refreshConnection();
            })
                .then(function (user) {
                resolve();
            })
                .catch(function (err) {
                if (err && (err.code === 403 || err.code === 410)) {
                    _this.miappLogout()
                        .then(function () {
                        reject({ code: 403, reason: 'Synchronization unauthorized : need to login again.' });
                    })
                        .catch(function () {
                        reject({ code: 403, reason: 'Synchronization unauthorized : need to login again.' });
                    });
                }
                else if (err && err.code) {
                    resolve();
                }
                else {
                    var errMessage = 'Error during syncronisation: ' + err.toString();
                    reject({ code: 500, reason: errMessage });
                }
            });
        });
    };
    InternalService.prototype.miappPutInDb = function (data) {
        var self = this;
        self.logger.log('miapp.sdk.service.miappPutInDb: ', data);
        if (!self.connection.getClientId() || !self.session.isReady()) {
            return self.promise.reject(new Error$1(401, 'DB put impossible. Need a user logged in.'));
        }
        var _id;
        if (data && typeof data === 'object' && Object.keys(data).indexOf('_id')) {
            _id = data._id;
        }
        if (!_id) {
            _id = self._generateObjectUniqueId(self.connection.miappId);
        }
        var crypto;
        if (self.connection.miappCrypto) {
            crypto = {
                obj: self.connection,
                method: 'encrypt'
            };
        }
        return self.session.put(data, _id, self.connection.getClientId(), self.sdk.org, self.connection.miappVersion, crypto);
    };
    InternalService.prototype.miappRemoveInDb = function (data_id) {
        var self = this;
        self.logger.log('miapp.sdk.service.miappRemoveInDb ', data_id);
        if (!self.session.isReady()) {
            return self.promise.reject(new Error$1(401, 'DB remove impossible. ' +
                'Need a user logged in.'));
        }
        if (!data_id || typeof data_id !== 'string') {
            return self.promise.reject(new Error$1(400, 'DB remove impossible. ' +
                'Need the data._id.'));
        }
        return self.session.remove(data_id);
    };
    InternalService.prototype.miappFindInDb = function (data_id) {
        var self = this;
        if (!self.connection.getClientId() || !self.session.isReady()) {
            return self.promise.reject(new Error$1(401, 'miapp.sdk.service.miappFindInDb : need a user logged in.'));
        }
        var crypto;
        if (self.connection.miappCrypto) {
            crypto = {
                obj: self.connection,
                method: 'decrypt'
            };
        }
        return self.session.get(data_id, crypto);
    };
    InternalService.prototype.miappFindAllInDb = function () {
        var self = this;
        if (!self.connection.getClientId() || !self.session.isReady()) {
            return self.promise.reject(new Error$1(401, 'Need a user logged in.'));
        }
        var crypto;
        if (self.connection.miappCrypto) {
            crypto = {
                obj: self.connection,
                method: 'decrypt'
            };
        }
        return self.session.getAll(crypto)
            .then(function (results) {
            self.connection.setCryptoSaltAsVerified();
            return self.promise.resolve(((results)));
        });
    };
    InternalService.prototype._loginInternal = function (login, password, updateProperties) {
        var self = this;
        self.logger.log('miapp.sdk.service._loginInternal');
        if (!self.connection.isReady()) {
            return self.promise.reject(new Error$1(403, 'Need an intialized MiappService'));
        }
        return new self.promise(function (resolve, reject) {
            self.connection.logout()
                .then(function () {
                return self.connection.getClient().login(login, password, updateProperties);
            })
                .catch(function (err) {
                return self.connection.getClient().login(login, password, updateProperties);
            })
                .then(function (loginUser) {
                loginUser.email = login;
                resolve(loginUser);
            })
                .catch(function (err) {
                self.logger.error('miapp.sdk.service._loginInternal error : ' + err);
                reject(err);
            });
        });
    };
    InternalService.prototype._removeAll = function () {
        this.connection.destroy();
        return this.session.destroy();
    };
    InternalService.prototype._createSession = function () {
        this.session.setRemote(this.connection.getDBs({ filter: 'theBestOnes' }));
        return this.session.create();
    };
    InternalService.prototype._testPromise = function (a) {
        if (a) {
            return this.promise.resolve('test promise ok ' + a);
        }
        return new this.promise(function (resolve, reject) {
            resolve('test promise ok');
        });
    };
    InternalService.prototype._generateObjectUniqueId = function (appName, type, name) {
        var now = new Date();
        var simpleDate = '' + now.getFullYear() + '' + now.getMonth() + '' + now.getDate()
            + '' + now.getHours() + '' + now.getMinutes();
        var sequId = ++InternalService._srvDataUniqId;
        var UId = '';
        if (appName && appName.charAt(0)) {
            UId += appName.charAt(0) + '';
        }
        if (type && type.length > 3) {
            UId += type.substring(0, 4);
        }
        if (name && name.length > 3) {
            UId += name.substring(0, 4);
        }
        UId += simpleDate + '' + sequId;
        return UId;
    };
    return InternalService;
}());
InternalService._srvDataUniqId = 0;
var MiappService = /** @class */ (function () {
    function MiappService() {
        this.logger = new LoggerService();
        this.promise = Promise;
        this.miappService = null;
    }
    MiappService.prototype.init = function (miappId, options) {
        if (!this.miappService) {
            this.miappService = new InternalService(this.logger, this.promise);
        }
        return this.miappService.miappInit(miappId, options);
    };
    MiappService.prototype.login = function (login, password) {
        if (!this.miappService) {
            return this.promise.reject(new Error$1(303, 'miapp.sdk.angular2.login : not initialized.'));
        }
        return this.miappService.miappLogin(login, password);
    };
    MiappService.prototype.loginAsDemo = function (options) {
        if (!this.miappService) {
            return this.promise.reject(new Error$1(303, 'miapp.sdk.angular2.loginAsDemo : not initialized.'));
        }
        return this.miappService.miappLoginInDemoMode(options);
    };
    MiappService.prototype.isLoggedIn = function () {
        if (!this.miappService) {
            return false;
        }
        return this.miappService.miappIsLogin();
    };
    MiappService.prototype.getRoles = function () {
        if (!this.miappService) {
            return [];
        }
        return this.miappService.miappRoles();
    };
    MiappService.prototype.getEndpoints = function () {
        if (!this.miappService) {
            return [];
        }
        return this.miappService.miappGetEndpoints();
    };
    MiappService.prototype.logout = function () {
        if (!this.miappService) {
            return this.promise.reject(new Error$1(303, 'miapp.sdk.angular2.logout : not initialized.'));
        }
        return this.miappService.miappLogout();
    };
    MiappService.prototype.sync = function (fnInitFirstData) {
        if (!this.miappService) {
            return this.promise.reject(new Error$1(401, 'miapp.sdk.angular2.sync : not initialized.'));
        }
        return this.miappService.miappSync(fnInitFirstData, this);
    };
    MiappService.prototype.put = function (data) {
        if (!this.miappService) {
            return this.promise.reject(new Error$1(401, 'miapp.sdk.angular2.put : not initialized.'));
        }
        return this.miappService.miappPutInDb(data);
    };
    MiappService.prototype.remove = function (id) {
        if (!this.miappService) {
            return this.promise.reject(new Error$1(401, 'miapp.sdk.angular2.remove : not initialized.'));
        }
        return this.miappService.miappRemoveInDb(id);
    };
    MiappService.prototype.find = function (id) {
        if (!this.miappService) {
            return this.promise.reject(new Error$1(401, 'miapp.sdk.angular2.find : not initialized.'));
        }
        return this.miappService.miappFindInDb(id);
    };
    MiappService.prototype.findAll = function () {
        if (!this.miappService) {
            return this.promise.reject(new Error$1(401, 'miapp.sdk.angular2.findAll : not initialized.'));
        }
        return this.miappService.miappFindAllInDb();
    };
    return MiappService;
}());
MiappService.decorators = [
    { type: core.Injectable },
];
MiappService.ctorParameters = function () { return []; };
var LoggerService = /** @class */ (function () {
    function LoggerService() {
    }
    LoggerService.prototype.log = function (message) {
    };
    LoggerService.prototype.error = function (message) {
        console.error(message);
    };
    LoggerService.prototype.warn = function (message) {
        console.warn(message);
    };
    return LoggerService;
}());
var MiappModule = /** @class */ (function () {
    function MiappModule() {
    }
    return MiappModule;
}());
MiappModule.decorators = [
    { type: core.NgModule, args: [{
                imports: [
                    common.CommonModule
                ],
                declarations: [],
                exports: [],
                providers: [MiappService]
            },] },
];
MiappModule.ctorParameters = function () { return []; };

exports.Base64 = Base64;
exports.LocalStorage = LocalStorage;
exports.Xor = Xor;
exports.MiappModule = MiappModule;
exports.MiappService = MiappService;
exports.LoggerService = LoggerService;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=miappio-sdk.umd.js.map
