import { LoggerInterface, ModuleServiceInterface, ModuleServiceInitOptionsInterface, ModuleServiceLoginOptionsInterface, ErrorInterface, EndpointInterface } from './interfaces';
/**
 * Angular2+ MiappService
 * @see ModuleServiceInterface
 *
 */
export declare class MiappService implements ModuleServiceInterface {
    private logger;
    private miappService;
    private promise;
    constructor();
    init(miappId: any, options?: ModuleServiceInitOptionsInterface): Promise<void | ErrorInterface>;
    login(login: any, password: any): Promise<any | ErrorInterface>;
    loginAsDemo(options?: ModuleServiceLoginOptionsInterface): Promise<any | ErrorInterface>;
    isLoggedIn(): boolean;
    getRoles(): Array<string>;
    getEndpoints(): Array<EndpointInterface>;
    logout(): Promise<void | ErrorInterface>;
    /**
     *
     * Synchronize DB
     * @param fnInitFirstData  a function with db as input and that return promise: call if DB is empty
     * @returns promise with this.session.db
     * @memberof miapp.angularService
     *
     * @example
     *  let initDb = function() {
     *     this.miappService.put('my first row');
     *  };
     *  this.miappService.sync(initDb)
     *  .then(user => ...)
     *  .catch(err => ...)
     *
     */
    sync(fnInitFirstData?: any): Promise<void | ErrorInterface>;
    /**
     * Store data in your session
     *
     * @param data to store
     * @returns
     */
    put(data: any): Promise<string | ErrorInterface>;
    /**
     * Find object Id and remove it from your session
     *
     * @param id of object to find and remove
     * @returns
     */
    remove(id: string): Promise<void | ErrorInterface>;
    /**
     * Find
     */
    find(id: string): Promise<any | ErrorInterface>;
    findAll(): Promise<any[] | ErrorInterface>;
}
export declare class LoggerService implements LoggerInterface {
    log(message: string): void;
    error(message: string): void;
    warn(message: string): void;
}
