export interface ErrorInterface {
    code: number;
    reason: string;
}
export interface EndpointInterface {
    key: string;
    uri: string;
}
/**
 * Interface used by all InternalService wrappers (angular.js, angular.io)
 *
 * @see MiappModule
 * @see MiappModule, MiappAngularService
 */
export interface ModuleServiceInterface {
    init(miappId: string, options?: ModuleServiceInitOptionsInterface): Promise<void | ErrorInterface>;
    login(login: string, password: string): Promise<any | ErrorInterface>;
    loginAsDemo(options?: ModuleServiceLoginOptionsInterface): Promise<any | ErrorInterface>;
    isLoggedIn(): boolean;
    getRoles(): Array<string>;
    getEndpoints(): Array<EndpointInterface>;
    logout(): Promise<void | ErrorInterface>;
    sync(fnInitFirstData?: any): Promise<any | ErrorInterface>;
    put(data: any): Promise<any | ErrorInterface>;
    remove(dataId: any): Promise<any | ErrorInterface>;
    find(id: string): Promise<any | ErrorInterface>;
    findAll(): Promise<any | ErrorInterface>;
}
export interface ModuleServiceInitOptionsInterface {
    prod: boolean;
    crypto?: boolean;
}
export interface ModuleServiceLoginOptionsInterface {
    accessToken?: string;
    idToken?: string;
    refreshToken?: string;
}
export interface SdkInterface {
    org: string;
    version: string;
    prod: boolean;
}
export interface LoggerInterface {
    log: (a?, b?, c?, d?, e?, f?) => any;
    error: (a?, b?, c?, d?, e?, f?) => any;
    warn: (a?, b?, c?, d?, e?, f?) => any;
}
