import { LoggerInterface, ModuleServiceInitOptionsInterface, ModuleServiceLoginOptionsInterface, ErrorInterface, EndpointInterface } from './interfaces';
/**
 * please use its angular.js or angular.io wrapper
 * usefull only for miapp.io dev team
 */
export declare class InternalService {
    private sdk;
    private logger;
    private promise;
    private storage;
    private session;
    private connection;
    constructor(logger: LoggerInterface, promise: PromiseConstructor);
    /**
     * Init connection & session
     * Check uri
     * Done each app start
     *
     * @param options Optional settings
     * @param options.miappId  required use your customized endpoints
     * @param options.miappSalt required use your customized endpoints
     * @param options.miappVersion required use your customized endpoints
     * @param options.devMode optional default false, use your customized endpoints
     * @returns
     */
    miappInit(miappId: string, options?: ModuleServiceInitOptionsInterface): Promise<void | ErrorInterface>;
    /**
     * Call it if miappIsLogin() === false
     * Erase all (db & storage)
     *
     * @param login
     * @param password
     * @returns
     */
    miappLogin(login: string, password: string): Promise<any | ErrorInterface>;
    /**
     *
     * @param options
     * @param options.accessToken optional
     * @param options.idToken  optional
     * @returns
     */
    miappLoginInDemoMode(options?: ModuleServiceLoginOptionsInterface): Promise<any | ErrorInterface>;
    miappGetEndpoints(options?: any): Array<EndpointInterface>;
    miappRoles(): any;
    miappMessage(): any;
    miappIsLogin(): boolean;
    /**
     *
     * @returns
     */
    miappLogout(): Promise<void | ErrorInterface>;
    /**
     * Synchronize DB
     *
     *
     * @param fnInitFirstData a function with db as input and that return promise: call if DB is empty
     * @param fnInitFirstData_Arg arg to set to fnInitFirstData()
     * @returns  promise
     */
    miappSync(fnInitFirstData?: any, fnInitFirstData_Arg?: any): Promise<void | ErrorInterface>;
    miappPutInDb(data: any): Promise<string | ErrorInterface>;
    miappRemoveInDb(data_id: string): Promise<void | ErrorInterface>;
    miappFindInDb(data_id: string): Promise<any | ErrorInterface>;
    miappFindAllInDb(): Promise<Array<any> | ErrorInterface>;
    /**
     * Logout then Login
     *
     * @param login
     * @param password
     * @param updateProperties
     */
    private _loginInternal(login, password, updateProperties?);
    protected _removeAll(): Promise<void | ErrorInterface>;
    private _createSession();
    private _testPromise(a?);
    private static _srvDataUniqId;
    private _generateObjectUniqueId(appName, type?, name?);
}
